/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev.h"
#include <string>

static gboolean on_delete_event (GtkWidget*, GdkEvent*, gpointer);
static gboolean on_auto_save (OpenLDev*);
static gboolean on_button_press_event_file_browser (GtkWidget*, GdkEventButton*, OpenLDev*);
static gboolean on_build_box_selected (GtkWidget*, GdkEventButton*, OpenLDev*);
static gboolean on_search_box_selected (GtkWidget*, GdkEventButton*, OpenLDev*);
static gboolean on_find_key (GtkWidget*, GdkEventKey*, OpenLDev*);
static gboolean on_goto_line_key (GtkWidget*, GdkEventKey*, OpenLDev*);
static void on_resize_window (GtkWidget*, GdkEvent*, OpenLDev*);
static void on_resize_pane (GtkWidget*, GdkEvent*, OpenLDev*);
static void on_close_toolbars (GtkWidget*, OpenLDev*);
static void on_goto_line (GtkWidget*, OpenLDev*);
static void on_find (GtkWidget*, OpenLDev*);
GtkWidget* show_splash_screen ();

// The magic all begins here...
int main (int argc, 
          char *argv[])
{
  if (argc > 1)
  {
    if (string (argv[1]) == "--help" || string (argv[1]) == "-?")
    {
      cout << "For help information, run 'man openldev'.\n";
      return EXIT_SUCCESS;
    }
    else if (string (argv[1]) == "--version")
    {
      string version = "OpenLDev " + string (VERSION)
                     + " (Fiordland)\nCopyright (C) 2004-2006 Andrew Krause\n";
      cout << version;
      return EXIT_SUCCESS;
    }
  }
  
  gtk_init (&argc, &argv);
  gnome_program_init ("openldev", VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_NONE);
  GtkWidget *splash = show_splash_screen ();
  
  // Create the OpenLDev window
  OpenLDev *openldev = OPENLDEV (openldev_new ());
  string icon = openldev_add_prefix ("/share/pixmaps/openldev/openldev24.png");
  gtk_window_set_icon_from_file (GTK_WINDOW (openldev), icon.c_str(), NULL);

  // Connect all of the main window signals
  g_signal_connect (G_OBJECT (openldev), "delete-event",
                    G_CALLBACK (on_delete_event), openldev);
  g_signal_connect (G_OBJECT (openldev), "event-after",
                    G_CALLBACK (on_resize_window), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->hpaned), "event-after",
                    G_CALLBACK (on_resize_pane), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->vpaned), "event-after",
                    G_CALLBACK (on_resize_pane), (gpointer) openldev);
  g_signal_connect (G_OBJECT (sidebar_get_file_browser (SIDEBAR (openldev->sidebar))), 
                    "button-press-event", G_CALLBACK (on_button_press_event_file_browser), 
                    (gpointer) openldev);
  g_signal_connect (G_OBJECT (OUTPUT (openldev->output)->search), "button-press-event", 
                    G_CALLBACK (on_search_box_selected), (gpointer) openldev);
  g_signal_connect (G_OBJECT (OUTPUT (openldev->output)->build), "button-press-event",
                    G_CALLBACK (on_build_box_selected), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->gototoolbar->button_close), "clicked",
                    G_CALLBACK (on_close_toolbars), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->gototoolbar->button_goto_line), "clicked",
                    G_CALLBACK (on_goto_line), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->findtoolbar->button_close), "clicked",
                    G_CALLBACK (on_close_toolbars), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->findtoolbar->button_find), "clicked",
                    G_CALLBACK (on_find), (gpointer) openldev);
  g_signal_connect (G_OBJECT (openldev->gototoolbar->goto_line), "key-press-event",
                    G_CALLBACK (on_goto_line_key), (gpointer) openldev);
  g_signal_connect (G_OBJECT (gtk_bin_get_child (GTK_BIN (openldev->findtoolbar->find_text))), 
                    "key-press-event", G_CALLBACK (on_find_key), (gpointer) openldev);
  
  // Load the last project or just set the directory to the home directory
  if (!openldev->env->last_project.empty() && openldev->env->auto_load)
    openldev_open_project (openldev, (gchar*) openldev->env->last_project.c_str());
  
  // Enable automatic saving if the user wants
  if (openldev->env->auto_save > 0)
  {
    gint time = openldev->env->auto_save * 1000 * 60;
    g_timeout_add (time, GSourceFunc (on_auto_save), (gpointer) openldev);
  }
  
  // Hide the splash screen & start the main look
  gtk_widget_destroy (splash);
  gtk_main();
	
  return EXIT_SUCCESS;
}

GtkWidget*
show_splash_screen ()
{
  string image_file = openldev_add_prefix ("/share/pixmaps/openldev/splash.png");
  GtkWidget *splash = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  GtkWidget *image = gtk_image_new_from_file (image_file.c_str());
  
  gtk_container_add (GTK_CONTAINER (splash), image);
  gtk_window_set_type_hint (GTK_WINDOW (splash), GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);
  gtk_window_set_position (GTK_WINDOW (splash), GTK_WIN_POS_CENTER_ALWAYS);
  gtk_window_set_keep_above (GTK_WINDOW (splash), TRUE);
  gtk_window_set_decorated (GTK_WINDOW (splash), FALSE);
  gtk_window_set_has_frame (GTK_WINDOW (splash), FALSE);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (splash), TRUE);
  gtk_widget_show_all (splash);
  
  // Make sure the splash screen is shown
  while (gtk_events_pending())
    gtk_main_iteration ();
  
  return splash;
}

static gboolean 
on_delete_event (GtkWidget *widget, 
                 GdkEvent *event, 
                 gpointer data)
{
  OpenLDev *openldev = OPENLDEV (data);
  
  gboolean is_modified = FALSE;
  for (int i=0; i < FILE_MANAGER (openldev->files)->tab_count; i++)
  {
    if (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[i])->modified)
      is_modified = TRUE;
  }
  
  if (!is_modified)
  {
    gtk_main_quit();
    return FALSE;
  }
  else if (openldev_question ("You have unsaved files! Are you.\nsure you want to close?"))
  {
    gtk_main_quit();
    return FALSE;
  }
  
  return TRUE;
}

// Autosave all of the files when called
static gboolean 
on_auto_save (OpenLDev *openldev)
{
  openldev_menu_file_save_all (openldev);
  return TRUE;
}

// When the window is resized, this checks the width and height and stores
// them so the same size can be restored on restart.
static void 
on_resize_window (GtkWidget *widget,
                  GdkEvent *event,
                  OpenLDev *openldev)
{  
  if (event->type == GDK_CONFIGURE)
  {
    // Get the current height & width and store them
    int width=0, height=0;
    width = widget->allocation.width;
    height = widget->allocation.height;
    openldev_environment_set_width (openldev->env, width);
    openldev_environment_set_height (openldev->env, height);
  }
}

// When one of the panes is resized, this checks the width and height of each
// pane so the same sizes can be restored on restart.
static void 
on_resize_pane (GtkWidget *widget,
                GdkEvent *event,
                OpenLDev *openldev)
{
  if (event->type == GDK_MOTION_NOTIFY)
  {   
    // Get the Project File & Output panes' width/height & store them
    openldev_environment_set_sidebar_width (openldev->env, 
             FILE_BROWSER (SIDEBAR (openldev->sidebar)->filebrowser)->swin->allocation.width);
    openldev_environment_set_outputmanager_height (openldev->env, 
             OUTPUT_BUILD (OUTPUT (openldev->output)->build)->swin->allocation.height);
  }
}

// Catch button press event & open popup menu or handle selection. This function
// receives the button event. If it is a right click, then the popup menu is
// shown. If it is a double click, the selected file is opened
static gboolean 
on_button_press_event_file_browser (GtkWidget *widget,
                                    GdkEventButton *event,
                                    OpenLDev *openldev)
{
  if((event->type == GDK_BUTTON_PRESS) && (event->button == 3))  
    gtk_menu_popup (GTK_MENU (openldev->menus->popup_filebrowser), 
                    NULL, NULL, NULL, (gpointer) openldev, 0, event->time);
  else if (event->type == GDK_2BUTTON_PRESS)
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
    GtkTreeModel *model;
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
      gchar *fn;
      gtk_tree_model_get (model, &iter, 1, &fn, -1);
   
      // If the text is not a group title, open it
      string file = g_strconcat (FILE_BROWSER (sidebar_get_file_browser 
                                 (SIDEBAR (openldev->sidebar)))->base,
                                 FILE_BROWSER (sidebar_get_file_browser
                                 (SIDEBAR (openldev->sidebar)))->current,
                                 fn, NULL);
      
      if (string(fn) == "..")
        file_browser_move_up (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (openldev->sidebar))));
      else if (g_file_test (file.c_str(), G_FILE_TEST_IS_DIR))
        file_browser_move (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (openldev->sidebar))), fn);
      else
        openldev_open_file (openldev, (gchar*) file.c_str());
      
      g_free (fn);
    }
  }
  else
    return FALSE;
  
  return TRUE;
}

// Catch the build box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and check if
// it is an error or warning. If it is, then the file containing the error or
// warning is opened. The cursor is then moved line of the error or warning
static gboolean 
on_build_box_selected (GtkWidget *widget, 
                       GdkEventButton *event, 
                       OpenLDev *openldev)
{
  if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))  
    gtk_menu_popup (GTK_MENU (openldev->menus->popup_buildoutput), 
                    NULL, NULL, NULL, (gpointer) openldev, 0, event->time);
  else if (event->type == GDK_2BUTTON_PRESS)
  {
    openldev_jump_to_build_output (GTK_MENU_ITEM (gtk_menu_item_new()), openldev);    
    return TRUE;
  }
  
  return FALSE;
}

// Catch the search box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and then the 
// file containing the search text is opened. The cursor is then moved line of 
// the search result
static gboolean 
on_search_box_selected (GtkWidget *widget,
                        GdkEventButton *event,
                        OpenLDev *openldev)
{
  if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))  
    gtk_menu_popup (GTK_MENU (openldev->menus->popup_searchoutput), 
                    NULL, NULL, NULL, (gpointer) openldev, 0, event->time);
  else if (event->type == GDK_2BUTTON_PRESS)
  {
    openldev_jump_to_search_output (GTK_MENU_ITEM (gtk_menu_item_new()), openldev);    
    return TRUE;
  }
  
  return FALSE;
}

// Close both toolbars. The find toolbar and the goto line toolbar.
static void 
on_close_toolbars (GtkWidget *widget, 
                   OpenLDev *openldev)
{
  gtk_widget_hide (openldev->gototoolbar->hbox);
  gtk_widget_hide (openldev->findtoolbar->hbox);
}

static void 
on_goto_line (GtkWidget *widget, 
              OpenLDev *openldev)
{ 
openldev_goto_line_number (openldev, -1); 
}

static void 
on_find (GtkWidget *widget, 
         OpenLDev *openldev)
{ 
  openldev_menu_edit_find_next (openldev); 
}

static gboolean 
on_find_key (GtkWidget *widget,
             GdkEventKey *event, 
             OpenLDev *openldev)
{ 
  if (event->type == GDK_KEY_PRESS)
    if (event->keyval == GDK_KP_Enter || event->keyval == GDK_Return)
      openldev_menu_edit_find_next (openldev);
  return FALSE;
}

static gboolean 
on_goto_line_key (GtkWidget *widget, 
                  GdkEventKey *event, 
                  OpenLDev *openldev)
{ 
  if (event->type == GDK_KEY_PRESS)
    if (event->keyval == GDK_KP_Enter || event->keyval == GDK_Return)
      openldev_goto_line_number (openldev, -1);
  return FALSE;
}
