/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-file.h"

#define NUM_MENUITEMS 17
gchar* menuitems[] = { "/MenuBar/File/FileSave", "/MenuBar/File/FileSaveAs", 
       "/MenuBar/File/FileSaveAll", "/MenuBar/File/FileRevert", "/MenuBar/File/FilePrint", 
       "/MenuBar/File/FilePrintPreview", "/MenuBar/Edit/EditUndo", "/MenuBar/Edit/EditRedo", 
       "/MenuBar/Edit/EditCut", "/MenuBar/Edit/EditCopy", "/MenuBar/Edit/EditPaste", 
       "/MenuBar/Edit/EditSelectAll", "/MenuBar/Edit/EditDeselect", "/ToolBar/FilePrint",
       "/ToolBar/FileSave", "/ToolBar/EditUndo", "/ToolBar/EditRedo" };

void openldev_on_delete_tab (GtkWidget*, gpointer);
void openldev_enable_project_menu_items (OpenLDev*);
void openldev_file_connect_signals (OpenLDev*);
void openldev_add_recent_file (OpenLDev*, const char*);
void openldev_upgrade_project (string&);

// Create a new file. This doesn't add the file to the project. The user must
// do that manually through the makefile editor.
void openldev_menu_file_new_file (OpenLDev *openldev)
{
  string file;
  NewFileDialog *dialog = openldev_new_file_dialog_new 
                          ((gchar*) openldev->project->location.c_str());
  openldev_new_file_dialog_run (dialog, file);
    
  if (file != "NULL" && file != "")
  {
    file_browser_refresh (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (openldev->sidebar))));
    openldev_open_file (openldev, (gchar*) file.c_str());     
  }
    
  gtk_widget_destroy (dialog->dialog);
  delete dialog;
}

// Create a new project by calling the new project dialog & then open the
// project that was created.
void openldev_menu_file_new_project (OpenLDev *openldev)
{
  NewProjectDialog *dialog = openldev_new_project_dialog_new ();
  GList *list = openldev_new_project_dialog_run (dialog);

  // Open newly created project if OK is pressed
  if (list != NULL)
  {
    string project_file = (const char*) g_list_nth_data (list, 0);
    string command = (const char*) g_list_nth_data (list, 1);
    
    gtk_notebook_set_current_page (GTK_NOTEBOOK (openldev->output), 0);
    gboolean result = output_build_run_command (OUTPUT_BUILD (OUTPUT 
                      (openldev->output)->build), command.c_str(), openldev->env);

    if (result)
      if (openldev_open_project (openldev, (gchar*) project_file.c_str()))
        openldev_menu_build_auto_generate (openldev);
  }
  
  delete dialog;
}

// Get the file that needs to be opened. This function doesn't actually open the
// file. Therefore, it is tricky and lies, but oh well.
void openldev_menu_file_open (OpenLDev *openldev)
{
  // Create file selector
  GtkWidget *dialog;
  dialog  = gtk_file_chooser_dialog_new ("Choose a project or File", 
                                         GTK_WINDOW (openldev),
                                         GTK_FILE_CHOOSER_ACTION_OPEN, 
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OPEN, GTK_RESPONSE_OK,
                                         NULL);
  
  // Set default folder
  if (file_manager_get_file_open (FILE_MANAGER (openldev->files)))
  {
    int l;
    string temp = file_manager_get_current_filename (FILE_MANAGER (openldev->files)), file;
    
    l = temp.find ("/", 0);
    while (l != string::npos)
    {
      file += temp.substr (0, l + 1);
      temp.erase (0, l + 1);
      l = temp.find ("/", 0);
    }
    
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), file.c_str());
  } 
  
  int result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
  {
    // If OK is pressed
    case (GTK_RESPONSE_OK):
    {
      // Check if file is a .dev file
      string filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      gtk_widget_destroy (dialog);
      bool result;
      
      if (filename.find (".dev", 0) != string::npos ||
          filename.find (".openldev", 0) != string::npos)
        result = openldev_open_project (openldev, (gchar*) filename.c_str());
      else
        result = openldev_open_file (openldev, (gchar*) filename.c_str());
      break;
    }
    default:
      gtk_widget_destroy (dialog);
      break;
  }
}

// Open the specified file if it exists. This function receives the filename that
// it is supposed to open. If a project is open, the file is added as the
// last opened file. The file is then opened, syntax highlighting is applied, 
// line numbering is calculated and displayed, signals are connected to the
// keystroke action and final initialization is applied
gboolean openldev_open_file (OpenLDev *openldev,
                             gchar *file)
{
  string filename = file;
  
  if (!g_file_test (filename.c_str(), G_FILE_TEST_EXISTS))
  {
    openldev_error ("Cannot open file. It does not exist!");
    if (!openldev->project->name.empty())
      openldev_project_settings_remove_lastfile (openldev->project, filename.c_str());
    return FALSE;
  }
  else
  {
    // Add a new tab to the notebookmanager
    if (file_manager_add_tab (FILE_MANAGER (openldev->files),
                             filename.c_str(),
                              openldev->env))
    {
      // Set the file as a last opened file
      if (!openldev->project->name.empty())
        openldev_project_settings_add_lastfile (openldev->project, filename.c_str());
  
      // Open file & read in text
      GtkSourceBuffer *buffer = file_manager_get_current_buffer (FILE_MANAGER (openldev->files));
      
      // Connect keystroke function & show/hide line numbering
      file_manager_set_properties (FILE_MANAGER (openldev->files), FALSE, openldev->env);
      openldev_file_connect_signals (openldev);
      
      // Set the font for the new file
      FileTab *tab = FILE_TAB (FILE_MANAGER (openldev->files)->tabs[FILE_MANAGER 
                               (openldev->files)->tab_count-1]);
      string font = openldev->env->font_face + " " + openldev->env->font_size;
      PangoFontDescription *fd = pango_font_description_from_string (font.c_str());
      PangoTabArray *tabs = file_manager_get_tab_array (fd, openldev->env, GTK_WIDGET (tab));
      gtk_text_view_set_tabs (GTK_TEXT_VIEW (tab), tabs);
      file_manager_modify_fonts (FILE_MANAGER (openldev->files), fd);
      openldev_manage_menu_items (openldev);
      openldev_add_recent_file (openldev, filename.c_str());
    }
  }
  
  return TRUE;
}

void openldev_add_recent_file (OpenLDev *openldev,
                               const char *filename)
{
  // Add a recent file & then repopulate them
  openldev_environment_add_recent_file (openldev->env, filename);
  
  for (int i=0; i < 5; i++)
  {
    string recent = "/MenuBar/File/FileRecent";
    gchar buff[2];
    sprintf (buff, "%d", i+1);
    recent += buff;
  
    if (g_slist_length (openldev->env->recent_files) <= i)
      gtk_widget_hide (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                       recent.c_str()));
    else
    {
      string name = string (buff) + ". " + string (openldev_environment_get_recent_short (openldev->env, i));
      gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN (gtk_ui_manager_get_widget 
                          (openldev->menus->uimanager, recent.c_str())))),
                          name.c_str());
      gtk_widget_show (gtk_ui_manager_get_widget (openldev->menus->uimanager, 
                       recent.c_str()));
    }
  }
}

// Connect the signals for the newly opened file
void openldev_file_connect_signals (OpenLDev *openldev)
{
  g_signal_connect (G_OBJECT (file_manager_get_current_view (FILE_MANAGER (openldev->files))), 
                    "key-press-event", G_CALLBACK (openldev_on_textview_keypress), 
                    (gpointer) openldev);
  g_signal_connect (G_OBJECT (file_manager_get_current_view (FILE_MANAGER (openldev->files))), 
                    "event", G_CALLBACK (openldev_on_textview_event), (gpointer) openldev);
  g_signal_connect (G_OBJECT (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[FILE_MANAGER 
                    (openldev->files)->tab_count-1])->button), "clicked",
                    G_CALLBACK (openldev_on_delete_tab), (gpointer) openldev);
}

// The project specified is opened. This function receives the filename of the
// project file. If the project was loaded, the last file opened is reloaded,
// the filelist is populated and the last project open is set to this project
gboolean openldev_open_project (OpenLDev *openldev,
                                gchar *file)
{
  string filename = file;
  if (filename.find (".openldev") != string::npos)
  {
    openldev_upgrade_project (filename);
    openldev_error ("<span weight='bold'>OpenLDev Message:</span>\n\nThis project file is no longer compatible with OpenLDev! New project files use the '.dev' extension.\n\nTherefore, the file has been upgraded. Please use the new file when opening your project in the future!");
  }

  file_manager_erase_all_tabs (FILE_MANAGER (openldev->files));
  gboolean success = openldev_project_settings_load_project_file (openldev->project, 
                                                                  filename.c_str());
 
  // If the project was opened
  if (success)
  {
    // Open the last file if it exists   
    for (unsigned int i=0; i < openldev->project->lastfiles.size(); i++)
      openldev_open_file (openldev, (gchar*) openldev->project->lastfiles[i].c_str());

    // Make environment ready for project
    openldev_populate_treeview (openldev);
    openldev_environment_set_last_project (openldev->env, filename.c_str());
    
    // Enable all of the project menu items because a project is now open
    openldev_enable_project_menu_items (openldev);
    g_signal_emit_by_name (G_OBJECT (openldev), "project-open", 0);
    return TRUE;
  }
  
  return FALSE;
}

// Enable all of the project menu items because a project is now open
void openldev_enable_project_menu_items (OpenLDev *openldev)
{
  menu_handle_set_sensitive (openldev->menus->uimanager, "/MenuBar/File/FileClose", TRUE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/MenuBar/Project", TRUE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/MenuBar/Build", TRUE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildCompileToolbar", TRUE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildBuildToolbar", TRUE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildExecute", TRUE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildClean", TRUE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildStop", TRUE);
}

// Save the current file & mark the file as unmodified. If the file does not
// have a name, then save as is called
void openldev_menu_file_save (OpenLDev *openldev)
{
  if (file_manager_get_file_open (FILE_MANAGER (openldev->files)))
  {
    int current = gtk_notebook_get_current_page (GTK_NOTEBOOK (openldev->files));
    
    if (IS_FILE_TAB (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[current])))
    {
      string current_fn = file_manager_get_current_filename (FILE_MANAGER (openldev->files));
      if (current_fn.empty() || current_fn == "EMPTY")
        openldev_menu_file_save_as (openldev);
  
      // Save the file & set as unmodified
      GtkSourceBuffer *buffer = file_manager_get_current_buffer (FILE_MANAGER (openldev->files));
      openldev_write_file (file_manager_get_current_filename (FILE_MANAGER (openldev->files)),
                           openldev_file_get_text_in_buffer (buffer));

      file_manager_mark_current_tab_modified (FILE_MANAGER (openldev->files), FALSE, current);
    }
  }
}

// Save the current file by a different name. The FileChooserDialog allows the
// user to choose the location and filename. Note: This function does not actually
// save the file, but rather just chooses a new name for it & calls on_file_save
void openldev_menu_file_save_as (OpenLDev *openldev)
{
  if (file_manager_get_file_open (FILE_MANAGER (openldev->files)))
  {
    int current = gtk_notebook_get_current_page (GTK_NOTEBOOK (openldev->files));
    
    if (IS_FILE_TAB (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[current])))
    {
      GtkWidget *dialog;
      dialog = gtk_file_chooser_dialog_new ("Save file as", 
                                            GTK_WINDOW (openldev),
                                            GTK_FILE_CHOOSER_ACTION_SAVE,
                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                            GTK_STOCK_SAVE, GTK_RESPONSE_OK,
                                            NULL);
      
      string temp = file_manager_get_current_filename (FILE_MANAGER (openldev->files)), file;
      int l = temp.find ("/", 0);
      while (l != string::npos)
      {
        file += temp.substr (0, l + 1);
        temp.erase (0, l + 1);
        l = temp.find ("/", 0);
      }

      // Add the filename to the text entry in Save As dialog
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), file.c_str());
      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog),
          file_manager_get_current_filename_short (FILE_MANAGER (openldev->files)).c_str());
  
      // Open fileselection dialog to get file location
      int result = gtk_dialog_run (GTK_DIALOG (dialog));
      switch (result)
      {
        // If OK is pressed
        case (GTK_RESPONSE_OK):
        {
          if (!openldev->project->name.empty())
          {
            openldev_project_settings_remove_lastfile (openldev->project, 
                     file_manager_get_current_filename (FILE_MANAGER (openldev->files)).c_str());
            openldev_project_settings_add_lastfile (openldev->project, 
                     gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog)));
          }
        
          file_manager_set_filename (FILE_MANAGER (openldev->files),
               gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog)));
          openldev_menu_file_save (openldev);
          break;
        }
        default:
          break;
      }
    
      gtk_widget_destroy (dialog);
    }
  }
}

// Call the notebookmanager function to save all of the files that are
// currently open
void openldev_menu_file_save_all (OpenLDev *openldev)
{
  for (int i=0; i < gtk_notebook_get_n_pages (GTK_NOTEBOOK (openldev->files)); i++)
  {
    // Save the file & set as unmodified
    if (IS_FILE_TAB (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[i])))
    {
      if (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[i])->modified)
      {
        GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW 
                                (FILE_MANAGER (openldev->files)->tabs[i]));
        openldev_write_file (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[i])->filename,
                             openldev_file_get_text_in_buffer (GTK_SOURCE_BUFFER (buffer)));

        file_manager_mark_current_tab_modified (FILE_MANAGER (openldev->files), FALSE, i);
      }
    }
  }
}

// If the user chooses to allow it, revert to the saved version of a file. Then
// the syntax highlighting, action manager and line numbering are refreshed
void openldev_menu_file_revert (OpenLDev *openldev)
{
  int current = gtk_notebook_get_current_page (GTK_NOTEBOOK (openldev->files));
    
  if (IS_FILE_TAB (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[current])))
    if (file_manager_get_file_open (FILE_MANAGER (openldev->files)))
      if (openldev_question ("Are you sure you want to reload this file?\nAny unsaved changes will be lost."))
        file_manager_revert_to_saved (FILE_MANAGER (openldev->files));
}

void openldev_on_delete_tab (GtkWidget *widget,
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;

  for (int i=0; i < FILE_MANAGER (openldev->files)->tab_count; i++)
    if (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[i])->button == widget)
      openldev_project_settings_remove_lastfile (openldev->project, 
                       FILE_TAB (FILE_MANAGER (openldev->files)->tabs[i])->filename);

  file_manager_on_delete_tab (widget, (gpointer) openldev->files);
  openldev_manage_menu_items (openldev);
}

void openldev_manage_menu_items (OpenLDev *openldev)
{
  GtkWidget *item;
  gboolean show = file_manager_get_file_open (FILE_MANAGER (openldev->files));
  item = gtk_ui_manager_get_widget (openldev->menus->uimanager, menuitems[0]);
  
  if (!(show == GTK_WIDGET_SENSITIVE(item)))
  {
    for (unsigned int i=0; i < NUM_MENUITEMS; i++)
    {
      item = gtk_ui_manager_get_widget (openldev->menus->uimanager, menuitems[i]);
      gtk_widget_set_sensitive (item, show);
    }
  }
}

/***
 Upgrade OpenLDev project files:
 Written by: Aaron Sebold
 ***/
 
#define XMLFILE_LINE_1		"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
#define XMLFILE_LINE_2		"<OpenLDevProject version=\"0.6+\">\n"
#define XMLFILE_LINE_LAST	"</OpenLDevProject>\n"

void openldev_upgrade_project (string &filename)
{
	gchar *buffer = NULL, *new_buffer = NULL, *current_line = NULL, 
	      *current_keyword = NULL, *need_free = NULL, *transform = NULL;
	gchar **buffer_lines = NULL;
	GList *keywords = NULL, *newwords = NULL, *defaults = NULL, *found = NULL;
	int x, y, z, first_case, compute, tf;

	/* Initialize the GLists */
	/* Keywords to look for */
	keywords = g_list_append (keywords, (void*) "template");
	keywords = g_list_append (keywords, (void*) "language");
	keywords = g_list_append (keywords, (void*) "compile");
	keywords = g_list_append (keywords, (void*) "build");
	keywords = g_list_append (keywords, (void*) "autogen");
	keywords = g_list_append (keywords, (void*) "configure");
	keywords = g_list_append (keywords, (void*) "clean");
	keywords = g_list_append (keywords, (void*) "execute");
	/* Keyword replacements */
	newwords = g_list_append (newwords, (void*) "ProjectTemplate");
	newwords = g_list_append (newwords, (void*) "Language");
	newwords = g_list_append (newwords, (void*) "Compile");
	newwords = g_list_append (newwords, (void*) "Build");
	newwords = g_list_append (newwords, (void*) "AutoGenerate");
	newwords = g_list_append (newwords, (void*) "Configure");
	newwords = g_list_append (newwords, (void*) "Clean");
	newwords = g_list_append (newwords, (void*) "Execute");
	/* If the keywords aren't found, here's their defaults */
	defaults = g_list_append (defaults, (void*) "console-c");
	defaults = g_list_append (defaults, (void*) "C");
	defaults = g_list_append (defaults, (void*) "make {FILE}");
	defaults = g_list_append (defaults, (void*) "make");
	defaults = g_list_append (defaults, (void*) "./autogen.sh");
	defaults = g_list_append (defaults, (void*) "./configure");
	defaults = g_list_append (defaults, (void*) "make clean");
	defaults = g_list_append (defaults, (void*) "{BASE}src/{NAME}");

	/* Let's bring the file in */
	buffer = (gchar*) openldev_read_file (filename).c_str();

	/* Split the file in to it's separate lines */
	buffer_lines = g_strsplit (buffer, "\n", 0);

	/* Add the first two common lines */
	new_buffer = g_strjoin ("", "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n", NULL);
	new_buffer = g_strjoin ("", new_buffer, "<OpenLDevProject version=\"0.6+\">\n", NULL);

	/* Parse and create the new file at the same time */
	x = 0;
	while (buffer_lines[x])
	{
		/* Get the current line */
		g_strstrip (current_line = g_strjoin ("", buffer_lines[x], NULL));

		/* Get the keyword on the current line */
		for (y = 0; y < g_list_length (keywords); y++)
		{
			current_keyword = g_strstr_len (current_line, strlen (current_line), 
			                                (gchar*) g_list_nth_data (keywords, y));
			if (current_keyword)
			{
				/* Found the keyword */
				int length = (int) strlen ((const char*) g_list_nth_data (keywords, y));
				need_free = g_strndup (current_keyword, length);
				current_keyword = g_strjoin ("", need_free, NULL);
				break;
			}
		}

		/* Didn't find a keyword */
		if (!current_keyword)
		{
			x++;
			continue;
		}

		/* Found a keyword */
		/* Make a note of the word */
		found = g_list_append (found, current_keyword);
		
		/* Transform the keyword in to XML format */
		transform = NULL;
		transform = g_strjoin ("", "  <", g_list_nth_data (newwords, y), ">", NULL);

		/* Use the rest of the line (kinda) */
		if (current_line[strlen (current_line) - 1] == '*')
		{
			compute = strlen (current_line) - (strlen (current_keyword) + 2);
			need_free = g_strreverse (current_line);
			need_free = g_strndup (need_free, compute);
			need_free = g_strreverse (need_free);
			need_free = g_strndup (need_free, (compute - 1));
		}
		else
		{
			compute = strlen (current_line) - (strlen (current_keyword) + 2);
			need_free = g_strreverse (current_line);
			need_free = g_strndup (need_free, compute);
			need_free = g_strreverse (need_free);
			need_free = g_strndup (need_free, (compute - (strlen (current_keyword) + 3)));
		}

    if (strcmp (current_keyword, "compile") == 0)
			transform = g_strjoin ("", transform, g_list_nth_data (defaults, y), NULL);
		else
			transform = g_strjoin ("", transform, need_free, NULL);

		/* Finish off the new XML format line */
		if (strcmp (current_keyword, "lastfiles") != 0)
			transform = g_strjoin ("", transform, "</", g_list_nth_data (newwords, y), ">", NULL);

		/* Add it to the new XML file's buffer */
		new_buffer = g_strjoin ("", new_buffer, transform, "\n", NULL);		

		x++;
	}

	/* Check for unused keywords, add the defaults if found */
	for (x = 0; x < g_list_length (keywords); x++)
	{
		tf = 0;
		for (y = 0; y < g_list_length (found); y++)
		{
			if (strcmp ((const char*) g_list_nth_data (found, y), 
			    (const char*) g_list_nth_data (keywords, x)) == 0)
			{
				tf = 1;
				break;
			}
		}

		/* Add the default to the new XML file as long as it isn't an open file */
		if (tf == 0 && strcmp ((const char*) g_list_nth_data (keywords, x), "lastfiles") != 0)
		{
			transform = NULL;
			transform = g_strjoin ("", "  <", g_list_nth_data (newwords, x), ">", NULL);
			transform = g_strjoin ("", transform, g_list_nth_data (defaults, x), NULL);
			transform = g_strjoin ("", transform, "</", g_list_nth_data (newwords, x), ">\n", NULL);
			new_buffer = g_strjoin ("", new_buffer, transform, NULL);
		}
	}

	/* Add the last common line */
	new_buffer = g_strjoin ("", new_buffer, "</OpenLDevProject>\n", NULL);
	filename.erase (filename.length()-9, 9);
	filename += ".dev";
	openldev_write_file (filename, new_buffer);

	/* Free unused variables (clean up) */
	g_free (need_free);
	g_list_free (keywords);
	g_list_free (newwords);
	g_list_free (defaults);
	g_list_free (found);
}
