/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-project.h"

void openldev_close_project (OpenLDev*);

// If a project is open, load the project information dialog.
void openldev_menu_project_info (OpenLDev *openldev)
{  
  if (openldev->project->name.empty()) 
    openldev_error ("Please open or create\na project first!");
  else 
  {
    ProjectInfoDialog *dialog = openldev_project_info_dialog_new (openldev->project);
    openldev_project_info_dialog_run (dialog);
  }
}

// If a project is open, load the project properties dialog and set the
// changes specified by the user. Also, rewrite the project compile flags to
// the src/Makefile.am
void openldev_menu_project_preferences (OpenLDev *openldev)
{  
  if (openldev->project->name.empty())
    openldev_error ("Please open or create\na project first!");
  else 
  {
    ProjectDialog *dialog = openldev_project_dialog_new (openldev->project);
    int result = openldev_project_dialog_run (dialog);
    
    if (result == GTK_RESPONSE_CLOSE)
      openldev->project = dialog->settings;
    
    gtk_widget_destroy (dialog->dialog);
    delete dialog;
  }
}

// Close the project, clear all of the files and then disable all of the project
// menu and toolbar items
void openldev_close_project (OpenLDev *openldev)
{
  openldev_project_settings_close_project (openldev->project);
  openldev_populate_treeview (openldev);
  openldev_menu_file_save_all (openldev);
  file_manager_erase_all_tabs (FILE_MANAGER (openldev->files));
  openldev_environment_set_last_project (openldev->env, "");
    
  menu_handle_set_sensitive (openldev->menus->uimanager, "/MenuBar/File/FileClose", FALSE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/MenuBar/Project", FALSE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/MenuBar/Build", FALSE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildCompileToolbar", FALSE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildBuildToolbar", FALSE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildExecute", FALSE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildClean", FALSE);
  menu_handle_set_sensitive (openldev->menus->uimanager, "/ToolBar/BuildStop", FALSE);
}

// Close project file. First call the close project funtion and then erase
// the project files treeview, save all of the files and erase all of the tabs.
void openldev_menu_file_close_project (OpenLDev *openldev)
{
  gboolean is_modified = FALSE;
  
  for (int i=0; i < FILE_MANAGER (openldev->files)->tab_count; i++)
  {
    if (FILE_TAB (FILE_MANAGER (openldev->files)->tabs[i])->modified)
      is_modified = TRUE;
  }
  
  if (!is_modified)
    openldev_close_project (openldev);
  else if (openldev_question ("You have unsaved files! Are you.\nsure you want to close?"))
    openldev_close_project (openldev);
}
