/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-project.h"

void openldev_project_settings_get_makefiles (vector<string>&, string);
vector<string> openldev_project_settings_get_subdirs (string);
vector<string> openldev_project_settings_get_makefile_sources (string);
string openldev_project_settings_load_version (string);

// Open the project located at the specified file name
ProjectSettings* openldev_project_settings_new (string filename)
{
  ProjectSettings* settings = new ProjectSettings ();

  if (openldev_project_settings_load_project_file (settings, filename))
    return settings;
  else
    return settings;
}

// Close the project by setting all of the values to empty strings, but
// make sure to write the project before closing.
void openldev_project_settings_close_project (ProjectSettings* settings)
{
  settings->lastfiles.clear();
  openldev_xml_document_close (&settings->xml);
  settings->template_name = "";
  settings->language = "";
  settings->compile = "";
  settings->build = "";
  settings->autogen = "";
  settings->configure = "";
  settings->clean = "";
  settings->location = "";
  settings->name = "";
  settings->version = "";
  settings->fullname = "";
}

// Load the project specified by the file name
gboolean 
openldev_project_settings_load_project_file (ProjectSettings* settings, 
                                             string file)
{
  string filename = file, element, attribute;
  if (!g_file_test (file.c_str(), G_FILE_TEST_EXISTS))
    return FALSE;
  
  settings->xml = openldev_xml_document_open (file);
  
  attribute = "version";
  if (openldev_xml_document_root_attribute (settings->xml, attribute) == "0.6+")
  {
    settings->fullname = file;
    settings->location = "/";
  
    filename.erase (0, 1);
    int l = filename.find ("/", 0);
    while (l != string::npos)
    {
      settings->location += filename.substr (0, l + 1);
      filename.erase (0, l + 1);
      l = filename.find ("/", 0);
    }

    // Get all of the attributes
    settings->name = filename.substr (0, filename.find (".dev", 0));
    settings->version = openldev_project_settings_load_version (settings->location + "configure.in");
    
    element = "OpenFile";
    settings->lastfiles = openldev_xml_document_element_values (settings->xml, element);
    
    element = "ProjectTemplate";
    settings->template_name = openldev_xml_document_element_value (settings->xml, element);
    element = "Language";
    settings->language = openldev_xml_document_element_value (settings->xml, element);
    element = "Compile";
    settings->compile = openldev_xml_document_element_value (settings->xml, element);
    element = "Build";
    settings->build = openldev_xml_document_element_value (settings->xml, element);
    element = "AutoGenerate";
    settings->autogen = openldev_xml_document_element_value (settings->xml, element);
    element = "Configure";
    settings->configure = openldev_xml_document_element_value (settings->xml, element);
    element = "Clean";
    settings->clean = openldev_xml_document_element_value (settings->xml, element);
    element = "Execute";
    settings->execute = openldev_xml_document_element_value (settings->xml, element);
  }
  else
  {
    openldev_error ("<span weight='bold'>OpenLDev Error:</span>\n\nThis project file format is not compatible with your version of OpenLDev!");
    return FALSE;
  }
  
  return TRUE;
}

// Check whether the specified file is in the project
gboolean openldev_project_settings_get_file_in_project (ProjectSettings *settings, 
                                                        const char *filename)
{
  string file = filename, location;
  while (file.find("/", 0) != string::npos)
  {
    location += file.substr (0, file.find("/", 0) + 1);
    file.erase (0, file.find("/", 0) + 1);
  }

  vector<string> sources = openldev_project_settings_get_makefile_sources (location + "Makefile.am");
  for (unsigned int i = 0; i < sources.size(); i++)
    if (sources[i] == file)
    {
      sources.clear ();
      return TRUE;
    }
  
  sources.clear ();
  return FALSE;
}

// Load the version from configure.in
string openldev_project_settings_load_version (string file)
{
  string text = openldev_read_file (file);
  int l1, l2 = text.find ("AM_INIT_AUTOMAKE", 0);
  
  l1 = text.find (",", l2) + 1;
  l2 = text.find (")", l1);
  
  text = text.substr (l1, l2-l1);
  while (text.find(" ", 0) != string::npos)
    text.erase (text.find(" ", 0), 1);
  
  return text; 
}

void openldev_project_settings_add_lastfile (ProjectSettings *settings,
                                             const char* value)
{ 
  for (unsigned int i=0; i < settings->lastfiles.size(); i++)
    if (settings->lastfiles[i] == value)
      return;

  string element = "OpenFile";
  settings->lastfiles.push_back (value);
  openldev_xml_document_add_element (settings->xml, element, value);
  openldev_xml_document_save (&settings->xml);
}

void openldev_project_settings_remove_lastfile (ProjectSettings *settings,
                                                const char* value)
{
  string element = "OpenFile";
  openldev_xml_document_remove_element (settings->xml, element, value); 
  openldev_xml_document_save (&settings->xml);

  vector<string>::iterator iter = settings->lastfiles.begin();
  for (unsigned int i=0; i < settings->lastfiles.size(); i++, iter++)
    if (settings->lastfiles[i] == value)
    {
      settings->lastfiles.erase (iter);
      return;
    }
}

void openldev_project_settings_set_language (ProjectSettings *settings,
                                             const char* value)
{ 
  string element = "Language";
  openldev_xml_document_set_element (settings->xml, element, value);
  openldev_xml_document_save (&settings->xml);
  settings->language = value;
}

void openldev_project_settings_set_compile (ProjectSettings *settings,
                                            const char* value)
{ 
  string element = "Compile";
  openldev_xml_document_set_element (settings->xml, element, value);
  openldev_xml_document_save (&settings->xml);
  settings->compile = value;
}

void openldev_project_settings_set_build (ProjectSettings *settings,
                                          const char* value)
{ 
  string element = "Build";
  openldev_xml_document_set_element (settings->xml, element, value);
  openldev_xml_document_save (&settings->xml);
  settings->build = value;
}

void openldev_project_settings_set_autogen (ProjectSettings *settings,
                                            const char* value)
{ 
  string element = "AutoGenerate";
  openldev_xml_document_set_element (settings->xml, element, value);
  openldev_xml_document_save (&settings->xml);
  settings->autogen = value;
}

void openldev_project_settings_set_configure (ProjectSettings *settings,
                                              const char* value)
{ 
  string element = "Configure";
  openldev_xml_document_set_element (settings->xml, element, value);
  openldev_xml_document_save (&settings->xml);
  settings->configure = value;
}

void openldev_project_settings_set_clean (ProjectSettings *settings,
                                          const char* value)
{ 
  string element = "Clean";
  openldev_xml_document_set_element (settings->xml, element, value);
  openldev_xml_document_save (&settings->xml);
  settings->clean = value;
}

void openldev_project_settings_set_execute (ProjectSettings *settings,
                                            const char* value)
{ 
  string element = "Execute";
  openldev_xml_document_set_element (settings->xml, element, value);
  openldev_xml_document_save (&settings->xml);
  settings->execute = value;
}

string openldev_project_settings_get_template (ProjectSettings *settings)
{ return settings->template_name.c_str();
}

vector<string> openldev_project_settings_get_lastfiles (ProjectSettings *settings)
{ return settings->lastfiles;
}

string openldev_project_settings_get_language (ProjectSettings *settings)
{ return settings->language.c_str();
}

string openldev_project_settings_get_compile (ProjectSettings *settings)
{ return settings->compile.c_str();
}

string openldev_project_settings_get_build (ProjectSettings *settings)
{ return settings->build.c_str();
}

string openldev_project_settings_get_autogen (ProjectSettings *settings)
{ return settings->autogen.c_str();
}

string openldev_project_settings_get_configure (ProjectSettings *settings)
{ return settings->configure.c_str();
}

string openldev_project_settings_get_clean (ProjectSettings *settings)
{ return settings->clean.c_str();
}

string openldev_project_settings_get_location (ProjectSettings *settings)
{ return settings->location.c_str();
}

string openldev_project_settings_get_name (ProjectSettings *settings)
{ return settings->name.c_str();
}

string openldev_project_settings_get_version (ProjectSettings *settings)
{ return settings->version.c_str();
}

string openldev_project_settings_get_fullname (ProjectSettings *settings)
{ return settings->fullname.c_str();
}

void openldev_project_settings_set_version (ProjectSettings* settings,
                                            const char* value)
{
  settings->version = value;
  
  string file = settings->location + "configure.in";
  string text = openldev_read_file (file);
  int l1, l2 = text.find ("AM_INIT_AUTOMAKE", 0);
  
  l1 = text.find (",", l2) + 1;
  l2 = text.find (")", l1);
  
  text.erase (l1, l2 - l1);
  text.insert (l1, " ");
  text.insert (l1 + 1, value);
  
  openldev_write_file (file, text);
}

// Get a list of all of the makefiles in the project
vector<string> openldev_project_settings_get_makefile_list (ProjectSettings *project)
{
  vector<string> makefiles;
  openldev_project_settings_get_makefiles (makefiles, project->location);
  
  return makefiles;
}

// Get a list of makefiles starting with the specified basedir. This is
// used recursively by the get_makefile_list function.
void openldev_project_settings_get_makefiles (vector<string> &makefiles,
                                              string base_dir)
{
  string makefile = base_dir + "Makefile.am";
  vector<string> subdirs = openldev_project_settings_get_subdirs (makefile);

  for (unsigned int i=0; i < subdirs.size(); i++)
    openldev_project_settings_get_makefiles (makefiles,
      (base_dir + subdirs.at(i) + "/"));

  makefiles.push_back (makefile);
  subdirs.clear ();
}

// Get a list of all of the sub directories in the project
vector<string> openldev_project_settings_get_subdirs (string makefile)
{
  Makefile *mf = openldev_makefile_new (makefile);
  vector<string> subdirs = openldev_makefile_get_subdirs (mf);
  delete mf;

  return subdirs;
}

// Get a list of all of the sources in the project
vector<string> openldev_project_settings_get_sources (ProjectSettings *project, 
                                                      const char *base_dir, 
                                                      gboolean subdirs)
{
  vector<string> makefiles, sources;
  
  // If sub-directories are to be included, get a list of all of the makefiles
  if (subdirs)
    openldev_project_settings_get_makefiles (makefiles, base_dir);
  // Otherwise just add the base directory
  else
    makefiles.push_back (string(base_dir) + "Makefile.am");
  
  for (unsigned int i=0; i < makefiles.size(); i++)
  {
    string makefile = makefiles[i].substr (0, makefiles[i].find("Makefile.am"));
    vector<string> temp = openldev_project_settings_get_makefile_sources (makefiles[i]);
    for (unsigned int j=0; j < temp.size(); j++)
    {        
      if (subdirs)
        sources.push_back (makefile + temp[j]);
      else
        sources.push_back (temp[j]);
    }
    temp.clear();
  }
  
  makefiles.clear();
  return sources;
}

// Get a list of all of the sources in the project
vector<string> openldev_project_settings_get_sources_all (ProjectSettings *project)
{ return openldev_project_settings_get_sources (project, project->location.c_str(), TRUE);
}

// Get a list of all of the sources in a specific makefile
vector<string> openldev_project_settings_get_makefile_sources (string file)
{
  Makefile *mf = openldev_makefile_new (file.c_str());
  vector<string> files = openldev_makefile_get_sources (mf);
  delete mf;
  
  return files;
}
