/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GOTOLINE_TOOLBAR_H
#define GOTOLINE_TOOLBAR_H

/*!
  \file openldev-goto-line-toolbar.h
  \brief GotoLineToolbar
*/

#include <gtk/gtk.h>

/*! \brief Goto line toolbar widgets including an error message */
struct GotoLineToolbar
{  
  /*! GtkLabel that tells the user what the toolbar does. */
  GtkWidget *label_goto_line;
  /*! GtkLabel error message. */
  GtkWidget *label_error;
  /*! GtkButton that is pressed to initiate the goto line action. */
  GtkWidget *button_goto_line;
  /*! GtkButton that is pressed to hide the toolbar. */
  GtkWidget *button_close;
  /*! GtkImage set to gtk-close. */
  GtkWidget *image_close;
  /*! GtkImage set to gtk-error. */
  GtkWidget *image_error;
  /*! GtkSpinButton that holds the user's preferred line */
  GtkWidget *goto_line;
  /*! GtkHBox that holds all of the goto line toolbar widgets. */
  GtkWidget *hbox;
};

// Internal function: Do not document!
GotoLineToolbar* openldev_goto_toolbar_new ();

/*! Show or hide the error message on the toolbar. When the line is out of bounds of the current file, this should be set to TRUE.
  \param toolbar A GotoLineToolbar widget.
  \param show TRUE if the error message should be shown.
*/
void openldev_goto_toolbar_show_error (GotoLineToolbar *toolbar, gboolean show);

/*! Get the line number the user wants to jump to. This can be in the range of 1 to 9999, so you must check the validity of the value and set the error message accordingly.
  \param toolbar A GotoLineToolbar widget.
  \return Line number the user wants to jump to.
*/
int openldev_goto_toolbar_get_goto_value (GotoLineToolbar *toolbar);

#endif
