/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-menu.h"

// Create the new menubar and toolbar handler
MenuHandle* 
menu_handle_new ()
{
  MenuHandle *handle = new MenuHandle();
  
  handle->group = gtk_action_group_new ("ActionGroup");
  handle->uimanager = gtk_ui_manager_new ();
  
  return handle;
}

// This function creates a new icon factory that adds the stock menu/toolbar
// items for the compile, build, project files and about commands
void 
menu_handle_create_icon_factory ()
{
  GtkIconFactory *icon_factory = gtk_icon_factory_new ();
  
  // Add new stock menu items
  openldev_add_new_stock_item (icon_factory, "Compile", "COMPILE",
      (gchar*) openldev_add_prefix ("/share/pixmaps/openldev/compile.png").c_str());
  openldev_add_new_stock_item (icon_factory, "Compile", "COMPILE_SMALL",
      (gchar*) openldev_add_prefix ("/share/pixmaps/openldev/compile_sm.png").c_str());
  openldev_add_new_stock_item (icon_factory, "Build", "BUILD", 
      (gchar*) openldev_add_prefix ("/share/pixmaps/openldev/build.png").c_str());
  openldev_add_new_stock_item (icon_factory, "Build", "BUILD_SMALL", 
      (gchar*) openldev_add_prefix ("/share/pixmaps/openldev/build_sm.png").c_str());
  openldev_add_new_stock_item (icon_factory, "New Project", "NEWPROJECT", 
      (gchar*) openldev_add_prefix ("/share/pixmaps/openldev/newproject.png").c_str());
  openldev_add_new_stock_item (icon_factory, "Project Info", "PROJECTINFO",
      (gchar*) openldev_add_prefix ("/share/pixmaps/openldev/projectinfo.png").c_str());
  
  // Add items to default
  gtk_icon_factory_add_default (icon_factory);
  icon_factory = NULL;
}

void
menu_handle_uimanager_init (MenuHandle* handle)
{
  GError *uierror;
  
  // Load the main menu, toolbar and popup menus
  if (!gtk_ui_manager_add_ui_from_file (handle->uimanager, 
      openldev_add_prefix ("/share/openldev/menus/mainmenu.ui").c_str(), &uierror))
    openldev_error ("<span weight='bold'>OpenLDev Error:</span>\n\nCannot load file <span style='italic'>mainmenu.ui</span>!");
  if (!gtk_ui_manager_add_ui_from_file (handle->uimanager, 
      openldev_add_prefix ("/share/openldev/menus/toolbar.ui").c_str(), &uierror))
    openldev_error ("<span weight='bold'>OpenLDev Error:</span>\n\nCannot load file <span style='italic'>toolbar.ui</span>!");
  if (!gtk_ui_manager_add_ui_from_file (handle->uimanager, 
      openldev_add_prefix ("/share/openldev/menus/popup_menus.ui").c_str(), &uierror))
    openldev_error ("<span weight='bold'>OpenLDev Error:</span>\n\nCannot load file <span style='italic'>popup_menus.ui</span>!");
  
  // Add the action group so signals can be connected
  gtk_ui_manager_insert_action_group (handle->uimanager, handle->group, 0);
  
  // Get all of the menu & toolbar widgets so they can easily be used
  handle->menubar = gtk_ui_manager_get_widget (handle->uimanager, "/MenuBar");
  handle->toolbar = gtk_ui_manager_get_widget (handle->uimanager, "/ToolBar");
  handle->popup_filebrowser = gtk_ui_manager_get_widget (handle->uimanager, "/FileBrowserMenu");
  handle->popup_searchoutput = gtk_ui_manager_get_widget (handle->uimanager, "/SearchOutputMenu");
  handle->popup_buildoutput = gtk_ui_manager_get_widget (handle->uimanager, "/BuildOutputMenu");
  
  // Set some important styles for the toolbar
  gtk_toolbar_set_style (GTK_TOOLBAR (handle->toolbar), GTK_TOOLBAR_BOTH_HORIZ);
  gtk_toolbar_set_show_arrow (GTK_TOOLBAR (handle->toolbar), FALSE);
  
  // Make sure the text on the New File and Open menu items is shown
  gtk_tool_item_set_is_important (GTK_TOOL_ITEM (gtk_ui_manager_get_widget 
                 (handle->uimanager, "/ToolBar/FileNewFileToolbar")), TRUE);
  gtk_tool_item_set_is_important (GTK_TOOL_ITEM (gtk_ui_manager_get_widget 
                 (handle->uimanager, "/ToolBar/FileOpenToolbar")), TRUE);
}

void 
menu_handle_set_sensitive (GtkUIManager *ui, 
                           string id, 
                           gboolean sensitive)
{
  gtk_widget_set_sensitive (gtk_ui_manager_get_widget (ui, id.c_str()), sensitive);
}

GtkWidget* 
menu_handle_get_menubar (MenuHandle* handle)
{ return handle->menubar;
}

GtkWidget* 
menu_handle_get_popup (MenuHandle* handle)
{ return handle->popup_filebrowser;
}

GtkWidget* 
menu_handle_get_toolbar (MenuHandle* handle)
{ return handle->toolbar;
}

GtkActionGroup* 
menu_handle_get_action_group (MenuHandle* handle)
{ return handle->group;
}

GtkUIManager* 
menu_handle_get_uimanager (MenuHandle* handle)
{ return handle->uimanager;
}
