/***************************************************************************
 *   Copyright (C) 2006 by Andrew Krause                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "insert-info.h"

void insert_info_add_menu_items (OpenLDev*);
void insert_info_user_name (GtkMenuItem*, gpointer);
void insert_info_real_name (GtkMenuItem*, gpointer);
void insert_info_home_directory (GtkMenuItem*, gpointer);
void insert_info_do_insertion (OpenLDev*, string);
void insert_info_gpl_header (GtkMenuItem*, gpointer);
void insert_info_date_and_time (GtkMenuItem*, gpointer);

DateAndTime *insert_info_parse_date (string);
string insert_info_get_date_string ();
gchar* create_date (DateAndTime*, string);

extern "C" 
{
// Initialize the plugin
G_MODULE_EXPORT PluginState init (gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  insert_info = new InsertInfoWidgets;
  insert_info_add_menu_items (openldev);
  
  return PLUGIN_OK;
} 

// This actually will just hide the plugin for later use
G_MODULE_EXPORT PluginState destroy (gpointer data)
{
  gtk_widget_hide (insert_info->menu_user);
  gtk_widget_hide (insert_info->item_user);
  gtk_widget_hide (insert_info->item_username);
  gtk_widget_hide (insert_info->item_realname);
  gtk_widget_hide (insert_info->item_homedir);
  gtk_widget_hide (insert_info->item_gpl);
  gtk_widget_hide (insert_info->item_date);
  gtk_widget_hide (insert_info->item_sep);
  
  return PLUGIN_OK;
}

// Show the plugin
G_MODULE_EXPORT PluginState reload (gpointer data)
{
  gtk_widget_show (insert_info->menu_user);
  gtk_widget_show (insert_info->item_user);
  gtk_widget_show (insert_info->item_username);
  gtk_widget_show (insert_info->item_realname);
  gtk_widget_show (insert_info->item_homedir);
  gtk_widget_show (insert_info->item_gpl);
  gtk_widget_show (insert_info->item_date);
  gtk_widget_show (insert_info->item_sep);
  
  return PLUGIN_OK;
}
};

// Add the menu items & connect signals for the Insert Info plugin
void insert_info_add_menu_items (OpenLDev *openldev)
{
  insert_info->menu_user = gtk_menu_new ();
  insert_info->item_user = gtk_menu_item_new_with_mnemonic ("Insert User Info ...");
  insert_info->item_username = gtk_menu_item_new_with_mnemonic ("Current _User Name");
  insert_info->item_realname = gtk_menu_item_new_with_mnemonic ("User's _Real Name");
  insert_info->item_homedir = gtk_menu_item_new_with_mnemonic ("_Home Directory");
  insert_info->item_gpl = gtk_menu_item_new_with_mnemonic ("Insert _GPL Header");
  insert_info->item_date = gtk_menu_item_new_with_mnemonic ("Insert Date and _Time");
  insert_info->item_sep = gtk_separator_menu_item_new ();
  GtkWidget *menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (gtk_ui_manager_get_widget 
                    (openldev->menus->uimanager, "/MenuBar/Edit")));
  
  gtk_menu_attach (GTK_MENU (insert_info->menu_user), 
                   insert_info->item_username, 0, 1, 0, 1);
  gtk_menu_attach (GTK_MENU (insert_info->menu_user), 
                   insert_info->item_realname, 0, 1, 1, 2);
  gtk_menu_attach (GTK_MENU (insert_info->menu_user), 
                   insert_info->item_homedir, 0, 1, 2, 3);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (insert_info->item_user), 
                             insert_info->menu_user);
  
  g_signal_connect (G_OBJECT (insert_info->item_username), "activate",
                    G_CALLBACK (insert_info_user_name), (gpointer) openldev);
  g_signal_connect (G_OBJECT (insert_info->item_realname), "activate",
                    G_CALLBACK (insert_info_real_name), (gpointer) openldev);
  g_signal_connect (G_OBJECT (insert_info->item_homedir), "activate",
                    G_CALLBACK (insert_info_home_directory), (gpointer) openldev);
  g_signal_connect (G_OBJECT (insert_info->item_gpl), "activate",
                    G_CALLBACK (insert_info_gpl_header), (gpointer) openldev);
  g_signal_connect (G_OBJECT (insert_info->item_date), "activate",
                    G_CALLBACK (insert_info_date_and_time), (gpointer) openldev);

  gtk_menu_attach (GTK_MENU (menu), insert_info->item_sep, 0, 1, 10, 11);
  gtk_menu_attach (GTK_MENU (menu), insert_info->item_user, 0, 1, 11, 12);
  gtk_menu_attach (GTK_MENU (menu), insert_info->item_date, 0, 1, 12, 13);
  gtk_menu_attach (GTK_MENU (menu), insert_info->item_gpl, 0, 1, 13, 14);
  
  gtk_widget_show (insert_info->menu_user);
  gtk_widget_show (insert_info->item_user);
  gtk_widget_show (insert_info->item_username);
  gtk_widget_show (insert_info->item_realname);
  gtk_widget_show (insert_info->item_homedir);
  gtk_widget_show (insert_info->item_gpl);
  gtk_widget_show (insert_info->item_date);
  gtk_widget_show (insert_info->item_sep);
}

// Do the actual insertion of whatever the text is
void insert_info_do_insertion (OpenLDev *openldev,
                               string text)
{
  gtk_text_buffer_insert_at_cursor (GTK_TEXT_BUFFER (file_manager_get_current_buffer
                                    (FILE_MANAGER (openldev->files))), 
                                    (gchar*) text.c_str(), text.length());
}

// Insert date and time
void insert_info_date_and_time (GtkMenuItem *menuitem,
                                gpointer data)
{ 
  DateAndTime *dat = insert_info_parse_date (insert_info_get_date_string ());
  string gladefile = string(INSTALL_PREFIX) + "/share/openldev/glade/inserttimedate.glade";
  GladeXML *xml = glade_xml_new (gladefile.c_str(), NULL, NULL);
  GtkWidget *dialog = glade_xml_get_widget (xml, "dialog");
  GtkComboBox *format = GTK_COMBO_BOX (glade_xml_get_widget (xml, "format"));
  
  gtk_combo_box_remove_text (format, 0);
  gtk_combo_box_append_text (format, create_date (dat, "%O %A, %Y %H:%M:%S"));
  gtk_combo_box_append_text (format, create_date (dat, "%D, %O %A, %Y %H:%M:%S"));
  gtk_combo_box_append_text (format, create_date (dat, "%d %A %o %Y %H:%M:%S %T"));
  gtk_combo_box_append_text (format, create_date (dat, "%m/%A/%Y"));
  gtk_combo_box_append_text (format, create_date (dat, "%m/%A/%Y %H:%M:%S"));
  gtk_combo_box_append_text (format, create_date (dat, "%Y-%m-%A %H:%M:%S"));
  gtk_combo_box_append_text (format, create_date (dat, "%d %o %A %H:%M:%S %T %Y"));
  gtk_combo_box_append_text (format, create_date (dat, "%d %o %A %H:%M:%S %Y"));
  gtk_combo_box_append_text (format, create_date (dat, "%d %A %o %Y %H:%M:%S %T"));
  gtk_combo_box_append_text (format, create_date (dat, "%d %A %o %Y %H:%M:%S"));
  gtk_combo_box_append_text (format, create_date (dat, "%A/%m/%Y"));
  gtk_combo_box_append_text (format, create_date (dat, "%A/%m/%y"));
  gtk_combo_box_append_text (format, create_date (dat, "%m/%A/%y"));
  gtk_combo_box_append_text (format, create_date (dat, "%D %A %O %Y"));
  gtk_combo_box_append_text (format, create_date (dat, "%D %O %A %Y"));
  gtk_combo_box_append_text (format, create_date (dat, "%Y-%m-%A"));
  gtk_combo_box_append_text (format, create_date (dat, "%A %O %Y"));
  gtk_combo_box_append_text (format, create_date (dat, "%O %A, %Y"));
  gtk_combo_box_append_text (format, create_date (dat, "%D %o %A"));
  gtk_combo_box_append_text (format, create_date (dat, "%H:%M:%S"));
  gtk_combo_box_append_text (format, create_date (dat, "%H:%M"));
  gtk_combo_box_append_text (format, create_date (dat, "%H.%M.%S"));
  gtk_combo_box_append_text (format, create_date (dat, "%H.%M"));
  gtk_combo_box_append_text (format, create_date (dat, "%A/%m/%Y %H:%M:%S"));
  gtk_combo_box_append_text (format, create_date (dat, "%A/%m/%y %H:%M:%S"));
  gtk_combo_box_append_text (format, create_date (dat, "%d, %A %o %Y %H:%M:%S"));
  gtk_combo_box_set_active (format, 0);
  
  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
    insert_info_do_insertion ((OpenLDev*) data, gtk_combo_box_get_active_text (format));
  
  gtk_widget_destroy (dialog);
  delete dat;
}

gchar* create_date (DateAndTime *dat,
                    string format)
{
  string final = format;
  int location;
  
  location = final.find ("%d", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->day_short);
  }
  
  location = final.find ("%D", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->day_full);
  }
  
  location = final.find ("%o", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->month_short);
  }
  
  location = final.find ("%O", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->month_full);
  }
  
  location = final.find ("%A", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->day);
  }
  
  location = final.find ("%H", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->hour);
  }
  
  location = final.find ("%M", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->minute);
  }
  
  location = final.find ("%S", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->second);
  }
  
  location = final.find ("%y", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->year_short);
  }
  
  location = final.find ("%Y", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->year_full);
  }
  
  location = final.find ("%T", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->timezone);
  }
  
  location = final.find ("%m", 0);
  if (location != (signed) string::npos) {
    final.erase (location, 2);
    final.insert (location, dat->month_int);
  }
  
  return (gchar*) final.c_str();
}

DateAndTime *insert_info_parse_date (string date)
{
  DateAndTime *dat = new DateAndTime;
 
  dat->day_short = date.substr (0, 3);
  dat->month_short = date.substr (4, 3);
  dat->day = date.substr (8, 2);
  dat->hour = date.substr (11, 2);
  dat->minute = date.substr (14, 2);
  dat->second = date.substr (17, 2);
  dat->timezone = date.substr (20, 3);
  dat->year_full = date.substr (24, 4);
  
  if (dat->day_short == "Tue")
    dat->day_full = "Tuesday";
  else if (dat->day_short == "Wed")
    dat->day_full = "Wednesday";
  else if (dat->day_short == "Thu")
    dat->day_full = "Thursday";
  else if (dat->day_short == "Sat")
    dat->day_full = "Saturday";
  else
    dat->day_full = dat->day_short + "day";

  if (dat->month_short == "Jan") {
    dat->month_full = "January";
    dat->month_int = "01";
  } else if (dat->month_short == "Feb") {
    dat->month_full = "February";
    dat->month_int = "02";
  } else if (dat->month_short == "Mar") {
    dat->month_full = "March";
    dat->month_int = "03";
  } else if (dat->month_short == "Apr") {
    dat->month_full = "April";
    dat->month_int = "04";
  } else if (dat->month_short == "May") {
    dat->month_full = "May";
    dat->month_int = "05";
  } else if (dat->month_short == "Jun") {
    dat->month_full = "June";
    dat->month_int = "06";
  } else if (dat->month_short == "Jul") {
    dat->month_full = "July";
    dat->month_int = "07";
  } else if (dat->month_short == "Aug") {
    dat->month_full = "August";
    dat->month_int = "08";
  } else if (dat->month_short == "Sep") {
    dat->month_full = "September";
    dat->month_int = "09";
  } else if (dat->month_short == "Oct") {
    dat->month_full = "October";
    dat->month_int = "10";
  } else if (dat->month_short == "Nov") {
    dat->month_full = "November";
    dat->month_int = "11";
  } else {
    dat->month_full = "December";
    dat->month_int = "12";
  }
  
  dat->year_short = dat->year_full.substr (2, 2);
  
  return dat;
}

// Insert user name
void insert_info_user_name (GtkMenuItem *menuitem,
                            gpointer data)
{ insert_info_do_insertion ((OpenLDev*) data, g_get_user_name ());
}

// Insert real name
void insert_info_real_name (GtkMenuItem *menuitem,
                            gpointer data)
{ insert_info_do_insertion ((OpenLDev*) data, g_get_real_name ());
}

// Insert home directory
void insert_info_home_directory (GtkMenuItem *menuitem,
                                 gpointer data)
{ insert_info_do_insertion ((OpenLDev*) data, g_get_home_dir ());
}

// Insert GPL Header
void insert_info_gpl_header (GtkMenuItem *menuitem,
                             gpointer data)
{ 
  string header = 
    "/***************************************************************************\n" \
    " *   Copyright (C) YYYY by                                                 *\n" \
    " *                                                                         *\n" \
    " *   This program is free software; you can redistribute it and/or modify  *\n" \
    " *   it under the terms of the GNU General Public License as published by  *\n" \
    " *   the Free Software Foundation; either version 2 of the License, or     *\n" \
    " *   (at your option) any later version.                                   *\n" \
    " *                                                                         *\n" \
    " *   This program is distributed in the hope that it will be useful,       *\n" \
    " *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *\n" \
    " *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *\n" \
    " *   GNU General Public License for more details.                          *\n" \
    " *                                                                         *\n" \
    " *   You should have received a copy of the GNU General Public License     *\n" \
    " *   along with this program; if not, write to the                         *\n" \
    " *   Free Software Foundation, Inc.,                                       *\n" \
    " *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *\n" \
    " ***************************************************************************/\n";

  // Get the full date
  int location = 0;
  string date = insert_info_get_date_string ();
  if (date.length () >= 24)
  {  
    date.erase (0, 24);
    location = header.find ("YYYY", 0);
    header.erase (location, 4);
    if (date.length () >= 4)
      header.insert (location, date.substr (0, 4));
  }
  
  // Insert username and remove traling spaces
  location += 8;
  string username = g_get_real_name ();
  header.erase (location, username.length());
  header.insert (location, username);
  
  insert_info_do_insertion ((OpenLDev*) data, header);
}

string insert_info_get_date_string ()
{
  gchar buff [8];
  string date;
  FILE *in = popen ("date", "r");
  while (fgets (buff, sizeof (buff), in))
    date += buff;
  pclose (in);
  
  return date;
}
